@@ -7,20 +7,22 @@ from typing import (
     Iterator,
     List,
     Optional,
+    Type,
     TypeVar,
     cast,
 )
 
-from streamable.wrappers import (
+from streamable.iters import (
     AsyncConcurrentMappingIterable,
     CatchingIterator,
     ConcurrentFlatteningIterable,
-    ConcurrentMappingIterable,
     FlatteningIterator,
+    GroupingByIterator,
     GroupingIterator,
     ObservingIterator,
     RaisingIterator,
-    SlowingIterator,
+    ThreadConcurrentMappingIterable,
+    ThrottlingIterator,
     TruncatingOnCountIterator,
     TruncatingOnPredicateIterator,
 )
@@ -29,26 +31,26 @@ T = TypeVar("T")
 U = TypeVar("U")
 
 from streamable import util
-
-
-class WrappedStopIteration(Exception):
-    pass
+from streamable.util import NoopStopIteration
 
 
 def catch(
     iterator: Iterator[T],
+    kind: Type[Exception] = Exception,
     when: Callable[[Exception], Any] = bool,
-    raise_after_exhaustion: bool = False,
+    finally_raise: bool = False,
 ) -> Iterator[T]:
-    when = util.reraise_as(when, source=StopIteration, target=WrappedStopIteration)
+    util.validate_iterator(iterator)
     return CatchingIterator(
         iterator,
+        kind,
         when,
-        raise_after_exhaustion=raise_after_exhaustion,
+        finally_raise=finally_raise,
     )
 
 
 def flatten(iterator: Iterator[Iterable[T]], concurrency: int = 1) -> Iterator[T]:
+    util.validate_iterator(iterator)
     util.validate_concurrency(concurrency)
     if concurrency == 1:
         return FlatteningIterator(iterator)
@@ -70,28 +72,29 @@ def group(
     seconds: float = float("inf"),
     by: Optional[Callable[[T], Any]] = None,
 ) -> Iterator[List[T]]:
+    util.validate_iterator(iterator)
     util.validate_group_size(size)
     util.validate_group_seconds(seconds)
-    if by is not None:
-        by = util.reraise_as(by, StopIteration, WrappedStopIteration)
     if size is None:
         size = cast(int, float("inf"))
-    return GroupingIterator(iterator, size, seconds, by)
+    if by is not None:
+        by = util.reraise_as(by, StopIteration, NoopStopIteration)
+        return GroupingByIterator(iterator, size, seconds, by)
+    return GroupingIterator(iterator, size, seconds)
 
 
 def map(
     transformation: Callable[[T], U], iterator: Iterator[T], concurrency: int = 1
 ) -> Iterator[U]:
+    util.validate_iterator(iterator)
     util.validate_concurrency(concurrency)
-    transformation = util.reraise_as(
-        transformation, StopIteration, WrappedStopIteration
-    )
+    transformation = util.reraise_as(transformation, StopIteration, NoopStopIteration)
     if concurrency == 1:
         return builtins.map(transformation, iterator)
     else:
         return RaisingIterator(
             iter(
-                ConcurrentMappingIterable(
+                ThreadConcurrentMappingIterable(
                     iterator,
                     transformation,
                     concurrency=concurrency,
@@ -106,12 +109,13 @@ def amap(
     iterator: Iterator[T],
     concurrency: int = 1,
 ) -> Iterator[U]:
+    util.validate_iterator(iterator)
     util.validate_concurrency(concurrency)
     return RaisingIterator(
         iter(
             AsyncConcurrentMappingIterable(
                 iterator,
-                util.reraise_as(transformation, StopIteration, WrappedStopIteration),
+                transformation,
                 buffer_size=concurrency,
             )
         )
@@ -119,12 +123,14 @@ def amap(
 
 
 def observe(iterator: Iterator[T], what: str) -> Iterator[T]:
+    util.validate_iterator(iterator)
     return ObservingIterator(iterator, what)
 
 
-def slow(iterator: Iterator[T], frequency: float) -> Iterator[T]:
-    util.validate_slow_frequency(frequency)
-    return SlowingIterator(iterator, frequency)
+def throttle(iterator: Iterator[T], per_second: float) -> Iterator[T]:
+    util.validate_iterator(iterator)
+    util.validate_throttle_per_second(per_second)
+    return ThrottlingIterator(iterator, per_second)
 
 
 def truncate(
@@ -132,6 +138,7 @@ def truncate(
     count: Optional[int] = None,
     when: Optional[Callable[[T], Any]] = None,
 ) -> Iterator[T]:
+    util.validate_iterator(iterator)
     util.validate_truncate_args(count, when)
     if count is not None:
         iterator = TruncatingOnCountIterator(iterator, count)
