@@ -1,3 +1,4 @@
+import logging
 from typing import (
     TYPE_CHECKING,
     Any,
@@ -11,6 +12,7 @@ from typing import (
     Optional,
     Sequence,
     Set,
+    Type,
     TypeVar,
     Union,
     cast,
@@ -18,12 +20,12 @@ from typing import (
 )
 
 from streamable.util import (
+    friendly_repr,
     get_logger,
     validate_concurrency,
     validate_group_seconds,
     validate_group_size,
-    validate_iterable,
-    validate_slow_frequency,
+    validate_throttle_per_second,
     validate_truncate_args,
 )
 
@@ -52,13 +54,6 @@ class Stream(Iterable[T]):
         Args:
             source (Union[Iterable[T], Callable[[], Iterable[T]]]): a source iterable or a function returning one (called for each new iteration on this stream).
         """
-        if not callable(source):
-            try:
-                validate_iterable(source)
-            except TypeError:
-                raise TypeError(
-                    "`source` must be either a Callable[[], Iterable] or an Iterable, but got a <class 'int'>"
-                )
         self._source = source
         self._upstream: "Optional[Stream]" = None
 
@@ -89,14 +84,8 @@ class Stream(Iterable[T]):
 
         return self.accept(IteratorVisitor[T]())
 
-    def exhaust(self) -> int:
-        """
-        Iterates over the stream until exhaustion and counts the elements yielded.
-
-        Returns:
-            int: The number of elements that have been yielded by the stream.
-        """
-        return sum(1 for _ in self)
+    def __repr__(self) -> str:
+        return f"Stream({friendly_repr(self.source)})"
 
     def accept(self, visitor: "Visitor[V]") -> V:
         """
@@ -106,36 +95,49 @@ class Stream(Iterable[T]):
 
     def catch(
         self,
+        kind: Type[Exception] = Exception,
         when: Callable[[Exception], Any] = bool,
-        raise_after_exhaustion: bool = False,
+        finally_raise: bool = False,
     ) -> "Stream[T]":
         """
-        Catches the upstream exceptions which are satisfying the provided `when` predicate.
+        Catches the upstream exceptions if they are instances of `kind` and they satisfy the `when` predicate.
 
         Args:
-            when (Callable[[Exception], Any], optional): The exception will be catched if `when(exception)` is Truthy (all exceptions catched by default).
-            raise_after_exhaustion (bool, optional): Set to True if you want the first catched exception to be raised when upstream is exhausted (default is False).
+            kind (Type[Exception], optional): The type of exceptions to catch (default is base Exception).
+            when (Callable[[Exception], Any], optional): An additional condition that must be satisfied (`when(exception)` must be Truthy) to catch the exception (always satisfied by default).
+            finally_raise (bool, optional): If True the first catched exception is raised when upstream's iteration ends (default is False).
 
         Returns:
             Stream[T]: A stream of upstream elements catching the eligible exceptions.
         """
-        return CatchStream(self, when, raise_after_exhaustion=raise_after_exhaustion)
+        return CatchStream(self, kind, when, finally_raise)
 
-    def explain(self) -> "Stream[T]":
+    def count(self) -> int:
         """
-        Logs this stream's explanation (INFO level)
-        """
-        get_logger().info("explanation:\n%s", self.explanation())
-        return self
+        Iterates over this stream until exhaustion and returns the count of elements.
 
-    def explanation(self) -> str:
-        """
         Returns:
-            str: A pretty representation of this stream's operations.
+            int: Number of elements yielded during an entire iteration over this stream.
         """
-        from streamable.visitors import explanation
 
-        return self.accept(explanation.ExplanationVisitor())
+        return sum(1 for _ in self)
+
+    def display(self, level: int = logging.INFO) -> "Stream[T]":
+        """
+        Logs (INFO level) a representation of the stream.
+
+        Args:
+            level (int, optional): The level of the log (default is INFO).
+
+        Returns:
+            Stream[T]: self.
+        """
+        import textwrap
+
+        get_logger().log(
+            level, "(\n%s\n)", textwrap.indent(repr(self).replace(" \\", ""), "    ")
+        )
+        return self
 
     def filter(self, keep: Callable[[T], Any] = bool) -> "Stream[T]":
         """
@@ -231,7 +233,7 @@ class Stream(Iterable[T]):
 
         Args:
             effect (Callable[[T], Any]): The function to be applied to each element as a side effect.
-            concurrency (int): The number of threads used to concurrently apply the `effect` (default is 1, meaning no concurrency).
+            concurrency (int): The number of threads used to concurrently apply the `effect` (default is 1, meaning no concurrency). Preserves the upstream order.
         Returns:
             Stream[T]: A stream of upstream elements, unchanged.
         """
@@ -249,7 +251,7 @@ class Stream(Iterable[T]):
 
         Args:
             effect (Callable[[T], Any]): The asynchronous function to be applied to each element as a side effect.
-            concurrency (int): How many asyncio tasks will run at the same time.
+            concurrency (int): How many asyncio tasks will run at the same time. Preserves the upstream order.
         Returns:
             Stream[T]: A stream of upstream elements, unchanged.
         """
@@ -291,7 +293,7 @@ class Stream(Iterable[T]):
 
         Args:
             transformation (Callable[[T], R]): The function to be applied to each element.
-            concurrency (int): The number of threads used to concurrently apply `transformation` (default is 1, meaning no concurrency).
+            concurrency (int): The number of threads used to concurrently apply `transformation` (default is 1, meaning no concurrency). Preserves the upstream order.
         Returns:
             Stream[R]: A stream of results of `transformation` applied to upstream elements.
         """
@@ -308,7 +310,7 @@ class Stream(Iterable[T]):
 
         Args:
             transformation (Callable[[T], Coroutine[Any, Any, U]]): The asynchronous function to be applied to each element.
-            concurrency (int): How many asyncio tasks will run at the same time.
+            concurrency (int): How many asyncio tasks will run at the same time. Preserves the upstream order.
         Returns:
             Stream[R]: A stream of results of `transformation` applied to upstream elements.
         """
@@ -330,22 +332,22 @@ class Stream(Iterable[T]):
             what (str): (plural) name representing the objects yielded.
 
         Returns:
-            Stream[T]: A stream of upstream elements whose iteration is logged for observability.
+            Stream[T]: A stream of upstream elements whose iteration's progress is logged.
         """
         return ObserveStream(self, what)
 
-    def slow(self, frequency: float) -> "Stream[T]":
+    def throttle(self, per_second: float) -> "Stream[T]":
         """
-        Slows the iteration down to a maximum `frequency`, more precisely an element will only be yielded if a period of `1/frequency` seconds has elapsed since the last yield.
+        Slows the iteration down to a maximum number of yields `per_second`, more precisely an element will only be yielded if a period of `1 / per_second` seconds has elapsed since the last yield.
 
         Args:
-            frequency (float): Maximum yields per second.
+            per_second (float): Maximum number of yields per second.
 
         Returns:
-            Stream[T]: A stream yielding upstream elements at a maximum `frequency`.
+            Stream[T]: A stream yielding upstream elements at a maximum `per_second`.
         """
-        validate_slow_frequency(frequency)
-        return SlowStream(self, frequency)
+        validate_throttle_per_second(per_second)
+        return ThrottleStream(self, per_second)
 
     def truncate(
         self, count: Optional[int] = None, when: Optional[Callable[[T], Any]] = None
@@ -386,58 +388,84 @@ class CatchStream(DownStream[T, T]):
     def __init__(
         self,
         upstream: Stream[T],
+        kind: Type[Exception],
         when: Callable[[Exception], Any],
-        raise_after_exhaustion: bool,
+        finally_raise: bool,
     ) -> None:
         super().__init__(upstream)
-        self.when = when
-        self.raise_after_exhaustion = raise_after_exhaustion
+        self._kind = kind
+        self._when = when
+        self._finally_raise = finally_raise
 
     def accept(self, visitor: "Visitor[V]") -> V:
         return visitor.visit_catch_stream(self)
 
+    def __repr__(self) -> str:
+        call = f"catch({friendly_repr(self._kind)}, when={friendly_repr(self._when)}, finally_raise={self._finally_raise})"
+        return f"{repr(self.upstream)} \\\n.{call}"
+
 
 class FilterStream(DownStream[T, T]):
     def __init__(self, upstream: Stream[T], keep: Callable[[T], Any]) -> None:
         super().__init__(upstream)
-        self.keep = keep
+        self._keep = keep
 
     def accept(self, visitor: "Visitor[V]") -> V:
         return visitor.visit_filter_stream(self)
 
+    def __repr__(self) -> str:
+        call = f"filter({friendly_repr(self._keep)})"
+        return f"{repr(self.upstream)} \\\n.{call}"
+
 
 class FlattenStream(DownStream[Iterable[T], T]):
     def __init__(self, upstream: Stream[Iterable[T]], concurrency: int) -> None:
         super().__init__(upstream)
-        self.concurrency = concurrency
+        self._concurrency = concurrency
 
     def accept(self, visitor: "Visitor[V]") -> V:
         return visitor.visit_flatten_stream(self)
 
+    def __repr__(self) -> str:
+        call = f"flatten(concurrency={self._concurrency})"
+        return f"{repr(self.upstream)} \\\n.{call}"
+
 
 class ForeachStream(DownStream[T, T]):
     def __init__(
         self, upstream: Stream[T], effect: Callable[[T], Any], concurrency: int
     ) -> None:
         super().__init__(upstream)
-        self.effect = effect
-        self.concurrency = concurrency
+        self._effect = effect
+        self._concurrency = concurrency
 
     def accept(self, visitor: "Visitor[V]") -> V:
         return visitor.visit_foreach_stream(self)
 
+    def __repr__(self) -> str:
+        call = (
+            f"foreach({friendly_repr(self._effect)}, concurrency={self._concurrency})"
+        )
+        return f"{repr(self.upstream)} \\\n.{call}"
+
 
 class AForeachStream(DownStream[T, T]):
     def __init__(
         self, upstream: Stream[T], effect: Callable[[T], Coroutine], concurrency: int
     ) -> None:
         super().__init__(upstream)
-        self.effect = effect
-        self.concurrency = concurrency
+        self._effect = effect
+        self._concurrency = concurrency
 
     def accept(self, visitor: "Visitor[V]") -> V:
         return visitor.visit_aforeach_stream(self)
 
+    def __repr__(self) -> str:
+        call = (
+            f"aforeach({friendly_repr(self._effect)}, concurrency={self._concurrency})"
+        )
+        return f"{repr(self.upstream)} \\\n.{call}"
+
 
 class GroupStream(DownStream[T, List[T]]):
     def __init__(
@@ -448,25 +476,33 @@ class GroupStream(DownStream[T, List[T]]):
         by: Optional[Callable[[T], Any]],
     ) -> None:
         super().__init__(upstream)
-        self.size = size
-        self.seconds = seconds
-        self.by = by
+        self._size = size
+        self._seconds = seconds
+        self._by = by
 
     def accept(self, visitor: "Visitor[V]") -> V:
         return visitor.visit_group_stream(self)
 
+    def __repr__(self) -> str:
+        call = f"group(size={self._size}, by={friendly_repr(self._by)}, seconds={self._seconds})"
+        return f"{repr(self.upstream)} \\\n.{call}"
+
 
 class MapStream(DownStream[T, U]):
     def __init__(
         self, upstream: Stream[T], transformation: Callable[[T], U], concurrency: int
     ) -> None:
         super().__init__(upstream)
-        self.transformation = transformation
-        self.concurrency = concurrency
+        self._transformation = transformation
+        self._concurrency = concurrency
 
     def accept(self, visitor: "Visitor[V]") -> V:
         return visitor.visit_map_stream(self)
 
+    def __repr__(self) -> str:
+        call = f"map({friendly_repr(self._transformation)}, concurrency={self._concurrency})"
+        return f"{repr(self.upstream)} \\\n.{call}"
+
 
 class AMapStream(DownStream[T, U]):
     def __init__(
@@ -476,29 +512,41 @@ class AMapStream(DownStream[T, U]):
         concurrency: int,
     ) -> None:
         super().__init__(upstream)
-        self.transformation = transformation
-        self.concurrency = concurrency
+        self._transformation = transformation
+        self._concurrency = concurrency
 
     def accept(self, visitor: "Visitor[V]") -> V:
         return visitor.visit_amap_stream(self)
 
+    def __repr__(self) -> str:
+        call = f"amap({friendly_repr(self._transformation)}, concurrency={self._concurrency})"
+        return f"{repr(self.upstream)} \\\n.{call}"
+
 
 class ObserveStream(DownStream[T, T]):
     def __init__(self, upstream: Stream[T], what: str) -> None:
         super().__init__(upstream)
-        self.what = what
+        self._what = what
 
     def accept(self, visitor: "Visitor[V]") -> V:
         return visitor.visit_observe_stream(self)
 
+    def __repr__(self) -> str:
+        call = f"""observe("{self._what}")"""
+        return f"{repr(self.upstream)} \\\n.{call}"
 
-class SlowStream(DownStream[T, T]):
-    def __init__(self, upstream: Stream[T], frequency: float) -> None:
+
+class ThrottleStream(DownStream[T, T]):
+    def __init__(self, upstream: Stream[T], per_second: float) -> None:
         super().__init__(upstream)
-        self.frequency = frequency
+        self._per_second = per_second
 
     def accept(self, visitor: "Visitor[V]") -> V:
-        return visitor.visit_slow_stream(self)
+        return visitor.visit_throttle_stream(self)
+
+    def __repr__(self) -> str:
+        call = f"throttle(per_second={self._per_second})"
+        return f"{repr(self.upstream)} \\\n.{call}"
 
 
 class TruncateStream(DownStream[T, T]):
@@ -509,8 +557,12 @@ class TruncateStream(DownStream[T, T]):
         when: Optional[Callable[[T], Any]] = None,
     ) -> None:
         super().__init__(upstream)
-        self.count = count
-        self.when = when
+        self._count = count
+        self._when = when
 
     def accept(self, visitor: "Visitor[V]") -> V:
         return visitor.visit_truncate_stream(self)
+
+    def __repr__(self) -> str:
+        call = f"truncate(count={self._count}, when={friendly_repr(self._when)})"
+        return f"{repr(self.upstream)} \\\n.{call}"
