@@ -1,4 +1,5 @@
 import asyncio
+import logging
 import time
 import timeit
 import unittest
@@ -19,7 +20,7 @@ from typing import (
 from parameterized import parameterized  # type: ignore
 
 from streamable import Stream
-from streamable.functions import WrappedStopIteration
+from streamable.functions import NoopStopIteration
 
 T = TypeVar("T")
 
@@ -136,13 +137,6 @@ class TestStream(unittest.TestCase):
             msg="The `upstream` attribute of a base Stream's instance must be None.",
         )
 
-        with self.assertRaisesRegex(
-            TypeError,
-            "`source` must be either a Callable\[\[\], Iterable\] or an Iterable, but got a <class 'int'>",
-            msg="Getting an Iterator from a Stream with a source not being a Union[Callable[[], Iterator], ITerable] must raise TypeError.",
-        ):
-            Stream(1)  # type: ignore
-
         self.assertIs(
             Stream(src)
             .group(100)
@@ -154,7 +148,7 @@ class TestStream(unittest.TestCase):
             .aforeach(async_identity)
             .catch()
             .observe()
-            .slow(1)
+            .throttle(1)
             .source,
             src,
             msg="`source` must be propagated by operations",
@@ -172,7 +166,7 @@ class TestStream(unittest.TestCase):
         ):
             Stream(src).upstream = Stream(src)  # type: ignore
 
-    def test_explanation(self) -> None:
+    def test_repr_and_display(self) -> None:
         class CustomCallable:
             pass
 
@@ -187,19 +181,13 @@ class TestStream(unittest.TestCase):
             .group(100)
             .observe("groups")
             .flatten(concurrency=4)
-            .slow(64)
+            .throttle(64)
             .observe("stream #1 elements")
-            .catch(lambda e: isinstance(e, (ValueError, TypeError)))
-        )
-        explanation_1 = complex_stream.explanation()
-        explanation_2 = complex_stream.explanation()
-        self.assertEqual(
-            explanation_1,
-            explanation_2,
-            msg="Stream.explain() must be deterministic.",
+            .catch(TypeError, finally_raise=True)
         )
+        explanation_1 = repr(complex_stream)
 
-        explanation_2 = complex_stream.map(str).explanation()
+        explanation_2 = repr(complex_stream.map(str))
         self.assertNotEqual(
             explanation_1,
             explanation_2,
@@ -208,13 +196,8 @@ class TestStream(unittest.TestCase):
 
         print(explanation_1)
 
-    def test_explain(self) -> None:
-        stream = Stream(src)
-        self.assertEqual(
-            stream.explain(),
-            stream,
-            msg="explain` should return self",
-        )
+        complex_stream.display()
+        complex_stream.display(logging.ERROR)
 
     def test_iter(self) -> None:
         self.assertIsInstance(
@@ -223,6 +206,20 @@ class TestStream(unittest.TestCase):
             msg="iter(stream) must return an Iterator.",
         )
 
+        with self.assertRaisesRegex(
+            TypeError,
+            "`source` must be either a Callable\[\[\], Iterable\] or an Iterable, but got a int",
+            msg="Getting an Iterator from a Stream with a source not being a Union[Callable[[], Iterator], ITerable] must raise TypeError.",
+        ):
+            iter(Stream(1))  # type: ignore
+
+        with self.assertRaisesRegex(
+            TypeError,
+            "`source` must be either a Callable\[\[\], Iterable\] or an Iterable, but got a Callable\[\[\], int\]",
+            msg="Getting an Iterator from a Stream with a source not being a Union[Callable[[], Iterator], ITerable] must raise TypeError.",
+        ):
+            iter(Stream(lambda: 1))  # type: ignore
+
     def test_add(self) -> None:
         from streamable.stream import FlattenStream
 
@@ -325,7 +322,7 @@ class TestStream(unittest.TestCase):
             ]
             for raised_exc, catched_exc in [
                 (TestError, TestError),
-                (StopIteration, (WrappedStopIteration, RuntimeError)),
+                (StopIteration, (NoopStopIteration, RuntimeError)),
             ]
             for concurrency in [1, 2]
             for method, throw_func_, throw_for_odd_func_ in [
@@ -346,18 +343,16 @@ class TestStream(unittest.TestCase):
     ) -> None:
         with self.assertRaises(
             catched_exc,
-            msg="At any concurrency, `map`and `foreach` must raise",
+            msg="At any concurrency, `map` and `foreach` and `amap` must raise",
         ):
             list(method(Stream(src), throw_func(raised_exc), concurrency))  # type: ignore
 
         self.assertListEqual(
             list(
-                method(Stream(src), throw_for_odd_func(raised_exc), concurrency).catch(  # type: ignore
-                    lambda exc: isinstance(exc, catched_exc)
-                )
+                method(Stream(src), throw_for_odd_func(raised_exc), concurrency).catch(catched_exc)  # type: ignore
             ),
             list(pair_src),
-            msg="At any concurrency, `map`and `foreach` must not stop after one exception occured.",
+            msg="At any concurrency, `map` and `foreach` and `amap` must not stop after one exception occured.",
         )
 
     @parameterized.expand(
@@ -448,41 +443,71 @@ class TestStream(unittest.TestCase):
 
     @parameterized.expand(
         [
-            [raised_exc, catched_exc, concurrency]
-            for raised_exc, catched_exc in [
+            [exception_type, mapped_exception_type, concurrency]
+            for exception_type, mapped_exception_type in [
                 (TestError, TestError),
-                (StopIteration, WrappedStopIteration),
+                (StopIteration, NoopStopIteration),
             ]
             for concurrency in [1, 2]
         ]
     )
     def test_flatten_with_exception(
         self,
-        raised_exc: Type[Exception],
-        catched_exc: Type[Exception],
+        exception_type: Type[Exception],
+        mapped_exception_type: Type[Exception],
         concurrency: int,
     ) -> None:
-        class odd_iterable(Iterable[int]):
-            def __init__(self, i, pair_exception: Type[Exception]) -> None:
-                self.i = i
-                self.pair_exception = pair_exception
+        n_iterables = 5
 
+        class IterableRaisingInIter(Iterable[int]):
             def __iter__(self) -> Iterator[int]:
-                if self.i % 2:
-                    raise self.pair_exception()
-                yield self.i
-
-        n_iterables = 4
+                raise exception_type
 
         self.assertSetEqual(
             set(
-                Stream(range(n_iterables))
-                .map(lambda i: cast(Iterable[int], odd_iterable(i, raised_exc)))
+                Stream(
+                    map(
+                        lambda i: (
+                            IterableRaisingInIter() if i % 2 else range(i, i + 1)
+                        ),
+                        range(n_iterables),
+                    )
+                )
                 .flatten(concurrency=concurrency)
-                .catch(catched_exc)
+                .catch(mapped_exception_type)
             ),
             set(range(0, n_iterables, 2)),
-            msg="At any concurrency the `flatten` method should be resilient to exceptions thrown by iterators, especially it should remap StopIteration one to WrappedStopIteration.",
+            msg="At any concurrency the `flatten` method should be resilient to exceptions thrown by iterators, especially it should remap StopIteration one to PacifiedStopIteration.",
+        )
+
+        class IteratorRaisingInNext(Iterator[int]):
+            def __init__(self) -> None:
+                self.first_next = True
+
+            def __iter__(self) -> Iterator[int]:
+                return self
+
+            def __next__(self) -> int:
+                if not self.first_next:
+                    raise StopIteration
+                self.first_next = False
+                raise exception_type
+
+        self.assertSetEqual(
+            set(
+                Stream(
+                    map(
+                        lambda i: (
+                            IteratorRaisingInNext() if i % 2 else range(i, i + 1)
+                        ),
+                        range(n_iterables),
+                    )
+                )
+                .flatten(concurrency=concurrency)
+                .catch(mapped_exception_type)
+            ),
+            set(range(0, n_iterables, 2)),
+            msg="At any concurrency the `flatten` method should be resilient to exceptions thrown by iterators, especially it should remap StopIteration one to PacifiedStopIteration.",
         )
 
     @parameterized.expand([[concurrency] for concurrency in [2, 4]])
@@ -575,24 +600,25 @@ class TestStream(unittest.TestCase):
         ):
             Stream(src).truncate(cast(int, float("inf")))
 
-        n_iterations = 0
         count = N // 2
         raising_stream_iterator = iter(
-            Stream(lambda: map(lambda x: x / 0, src)).truncate(count)
-        )
-        while True:
-            try:
-                next(raising_stream_iterator)
-            except ZeroDivisionError:
-                n_iterations += 1
-            except StopIteration:
-                break
-        self.assertEqual(
-            n_iterations,
-            count,
-            msg="`truncate` must not stop iteration when encountering exceptions",
+            Stream(lambda: map(lambda x: round((1 / x) * x**2), src)).truncate(count)
         )
 
+        with self.assertRaises(
+            ZeroDivisionError,
+            msg="`truncate` must not stop iteration when encountering exceptions and raise them without counting them...",
+        ):
+            next(raising_stream_iterator)
+
+        self.assertEqual(list(raising_stream_iterator), list(range(1, count + 1)))
+
+        with self.assertRaises(
+            StopIteration,
+            msg="... and after reaching the limit it still continues to raise StopIteration on calls to next",
+        ):
+            next(raising_stream_iterator)
+
         iter_truncated_on_predicate = iter(Stream(src).truncate(when=lambda n: n == 5))
         self.assertEqual(
             list(iter_truncated_on_predicate),
@@ -683,16 +709,25 @@ class TestStream(unittest.TestCase):
         self.assertListEqual(
             list(
                 Stream(lambda: map(slow_identity, src)).group(
-                    size=100, seconds=0.9 * slow_identity_duration
+                    size=100, seconds=slow_identity_duration / 1000
                 )
             ),
             list(map(lambda e: [e], src)),
-            msg="`group` should yield each upstream element alone in a single-element group if `seconds` inferior to the upstream yield period",
+            msg="`group` should not yield empty groups even though `seconds` if smaller than upstream's frequency",
         )
         self.assertListEqual(
             list(
                 Stream(lambda: map(slow_identity, src)).group(
-                    size=100, seconds=1.8 * slow_identity_duration
+                    size=100, seconds=slow_identity_duration / 1000, by=lambda _: None
+                )
+            ),
+            list(map(lambda e: [e], src)),
+            msg="`group` with `by` argument should not yield empty groups even though `seconds` if smaller than upstream's frequency",
+        )
+        self.assertListEqual(
+            list(
+                Stream(lambda: map(slow_identity, src)).group(
+                    size=100, seconds=2 * slow_identity_duration
                 )
             ),
             list(map(lambda e: [e, e + 1], pair_src)),
@@ -771,7 +806,7 @@ class TestStream(unittest.TestCase):
             msg="`group` should yield incomplete groups when `by` raises",
         )
         with self.assertRaises(
-            WrappedStopIteration,
+            NoopStopIteration,
             msg="`group` should raise and skip `elem` if `by(elem)` raises",
         ):
             next(stream_iter)
@@ -781,17 +816,17 @@ class TestStream(unittest.TestCase):
             msg="`group` should continue yielding after `by`'s exception has been raised.",
         )
 
-    def test_slow(self) -> None:
+    def test_throttle(self) -> None:
         # behavior with invalid arguments
-        for frequency in [-0.01, 0]:
+        for per_second in [-0.01, 0]:
             with self.assertRaises(
                 ValueError,
-                msg="`slow` should raise error when called with `frequency` <= 0.",
+                msg="`throttle` should raise error when called with `per_second` <= 0.",
             ):
-                list(Stream([1]).slow(frequency=frequency))
+                list(Stream([1]).throttle(per_second=per_second))
 
-        frequency = 3
-        period = 1 / frequency
+        per_second = 3
+        period = 1 / per_second
         super_slow_elem_pull_seconds = 1
         N = 10
         expected_duration = (N - 1) * period + super_slow_elem_pull_seconds
@@ -800,7 +835,7 @@ class TestStream(unittest.TestCase):
             .foreach(
                 lambda e: time.sleep(super_slow_elem_pull_seconds) if e == 0 else None
             )
-            .slow(frequency=frequency)
+            .throttle(per_second=per_second)
         )
         self.assertAlmostEqual(
             duration,
@@ -809,21 +844,21 @@ class TestStream(unittest.TestCase):
             msg="avoid bursts after very slow particular upstream elements",
         )
 
-        # float or int frequency
-        for frequency in [0.9, 2, 50]:
+        # float or int per_second
+        for per_second in [0.9, 2, 50]:
             self.assertEqual(
                 next(
                     iter(
                         Stream(src)
-                        .slow(frequency=frequency)
-                        .slow(frequency=frequency * 2)
+                        .throttle(per_second=per_second)
+                        .throttle(per_second=per_second * 2)
                     )
                 ),
                 0,
-                msg="`slow` should avoid 'ValueError: sleep length must be non-negative'",
+                msg="`throttle` should avoid 'ValueError: sleep length must be non-negative'",
             )
 
-            stream_iterator = iter(Stream(src).slow(frequency=frequency))
+            stream_iterator = iter(Stream(src).throttle(per_second=per_second))
             start_time = time.time()
             a = next(stream_iterator)
             b = next(stream_iterator)
@@ -831,25 +866,39 @@ class TestStream(unittest.TestCase):
             end_time = time.time()
 
             self.assertEqual(
-                a, 0, msg="`slow` must forward upstream elements unchanged"
+                a, 0, msg="`throttle` must forward upstream elements unchanged"
             )
             self.assertEqual(
-                b, 1, msg="`slow` must forward upstream elements unchanged"
+                b, 1, msg="`throttle` must forward upstream elements unchanged"
             )
             self.assertEqual(
-                c, 2, msg="`slow` must forward upstream elements unchanged"
+                c, 2, msg="`throttle` must forward upstream elements unchanged"
             )
 
-            period = 1 / frequency
+            period = 1 / per_second
             expected_duration = 3 * period
             self.assertAlmostEqual(
                 end_time - start_time,
                 expected_duration,
                 delta=0.3 * expected_duration,
-                msg="`slow` must respect the frequency set.",
+                msg="`throttle` must respect the per_second set.",
             )
 
     def test_catch(self) -> None:
+        self.assertEqual(
+            list(Stream(src).catch(finally_raise=True)),
+            list(src),
+            msg="`catch` should yield elements in exception-less scenarios",
+        )
+        with self.assertRaisesRegex(
+            TypeError,
+            "`iterator` should be an Iterator, but got a <class 'list'>",
+            msg="`catch` function should raise TypError when first argument is not an Iterator",
+        ):
+            from streamable.functions import catch
+
+            catch(cast(Iterator[int], [3, 4]))
+
         def f(i):
             return i / (3 - i)
 
@@ -857,9 +906,9 @@ class TestStream(unittest.TestCase):
         safe_src = list(src)
         del safe_src[3]
         self.assertListEqual(
-            list(stream.catch(lambda e: isinstance(e, ZeroDivisionError))),
+            list(stream.catch(ZeroDivisionError)),
             list(map(f, safe_src)),
-            msg="If the exception type matches the `when`, then the impacted element should be ignored.",
+            msg="If the exception type matches the `kind`, then the impacted element should be ignored.",
         )
         self.assertListEqual(
             list(stream.catch()),
@@ -871,7 +920,7 @@ class TestStream(unittest.TestCase):
             ZeroDivisionError,
             msg="If a non catched exception type occurs, then it should be raised.",
         ):
-            list(stream.catch(lambda e: False))
+            list(stream.catch(TestError))
 
         first_value = 1
         second_value = 2
@@ -888,10 +937,8 @@ class TestStream(unittest.TestCase):
 
         erroring_stream: Stream[int] = Stream(lambda: map(lambda f: f(), functions))
         for catched_erroring_stream in [
-            erroring_stream.catch(raise_after_exhaustion=True),
-            erroring_stream.catch(
-                lambda e: isinstance(e, Exception), raise_after_exhaustion=True
-            ),
+            erroring_stream.catch(finally_raise=True),
+            erroring_stream.catch(Exception, finally_raise=True),
         ]:
             erroring_stream_iterator = iter(catched_erroring_stream)
             self.assertEqual(
@@ -902,13 +949,13 @@ class TestStream(unittest.TestCase):
             n_yields = 1
             with self.assertRaises(
                 TestError,
-                msg="`catch` should raise the first error encountered when `raise_after_exhaustion` is True.",
+                msg="`catch` should raise the first error encountered when `finally_raise` is True.",
             ):
                 for _ in erroring_stream_iterator:
                     n_yields += 1
             with self.assertRaises(
                 StopIteration,
-                msg="`catch` with `raise_after_exhaustion`=True should finally raise StopIteration to avoid infinite recursion if there is another catch downstream.",
+                msg="`catch` with `finally_raise`=True should finally raise StopIteration to avoid infinite recursion if there is another catch downstream.",
             ):
                 next(erroring_stream_iterator)
             self.assertEqual(
@@ -917,11 +964,9 @@ class TestStream(unittest.TestCase):
                 msg="3 elements should have passed been yielded between catched exceptions.",
             )
 
-        only_catched_errors_stream = (
-            Stream(range(2000))
-            .map(lambda i: throw(TestError))
-            .catch(lambda e: isinstance(e, TestError))
-        )
+        only_catched_errors_stream = Stream(
+            map(lambda _: throw(TestError), range(2000))
+        ).catch(TestError)
         self.assertEqual(
             list(only_catched_errors_stream),
             [],
@@ -933,20 +978,52 @@ class TestStream(unittest.TestCase):
         ):
             next(iter(only_catched_errors_stream))
 
+        iterator = iter(
+            Stream(map(throw, [TestError, ValueError]))
+            .catch(ValueError, finally_raise=True)
+            .catch(TestError, finally_raise=True)
+        )
+        with self.assertRaises(
+            ValueError,
+            msg="With 2 chained `catch`s with `finally_raise=True`, the error catched by the first `catch` is finally raised first (even though it was raised second)...",
+        ):
+            next(iterator)
+        with self.assertRaises(
+            TestError,
+            msg="... and then the error catched by the second `catch` is raised...",
+        ):
+            next(iterator)
+        with self.assertRaises(
+            StopIteration,
+            msg="... and a StopIteration is raised next.",
+        ):
+            next(iterator)
+
+        with self.assertRaises(
+            TypeError,
+            msg="`catch` does not catch if `when` not satisfied",
+        ):
+            list(
+                Stream(map(throw, [ValueError, TypeError])).catch(
+                    Exception, when=lambda exception: "ValueError" in repr(exception)
+                )
+            )
+
     def test_observe(self) -> None:
         value_error_rainsing_stream: Stream[List[int]] = (
-            Stream("123--567")
-            .slow(1)
+            Stream("123--678")
+            .throttle(10)
             .observe("chars")
             .map(int)
             .observe("ints")
             .group(2)
             .observe("int pairs")
+            # .foreach(lambda g: print("end", g))
         )
 
         self.assertListEqual(
             list(value_error_rainsing_stream.catch(ValueError)),
-            [[1, 2], [3], [5, 6], [7]],
+            [[1, 2], [3], [6, 7], [8]],
             msg="This can break due to `group`/`map`/`catch`, check other breaking tests to determine quickly if it's an issue with `observe`.",
         )
 
@@ -959,20 +1036,21 @@ class TestStream(unittest.TestCase):
     def test_is_iterable(self) -> None:
         self.assertIsInstance(Stream(src), Iterable)
 
-    def test_exhaust(self) -> None:
+    def test_count(self) -> None:
         l: List[int] = []
 
         def effect(x: int) -> None:
             nonlocal l
             l.append(x)
 
-        self.assertEqual(
-            Stream(lambda: map(effect, src)).exhaust(),
+        stream = Stream(lambda: map(effect, src))
+        self.assertIs(
+            stream.count(),
             N,
-            msg="`__len__` should return the number of iterated elements.",
+            msg="`exhaust` should return self.",
         )
         self.assertListEqual(
-            l, list(src), msg="`__len__` should iterate over the entire stream."
+            l, list(src), msg="`exhaust` should iterate over the entire stream."
         )
 
     def test_multiple_iterations(self) -> None:
